/**
 **/
#include <stdio.h>

#include "drop.h"

#define IMAGE_DIRECTORY_ENTRY_BASERELOC       5   // Base Relocation Table

#define IMAGE_REL_I386_ABSOLUTE         0x0000  // Reference is absolute, no relocation is necessary
#define IMAGE_REL_I386_DIR16            0x0001  // Direct 16-bit reference to the symbols virtual address
#define IMAGE_REL_I386_REL16            0x0002  // PC-relative 16-bit reference to the symbols virtual address
#define IMAGE_REL_I386_DIR32            0x0006  // Direct 32-bit reference to the symbols virtual address
#define IMAGE_REL_I386_DIR32NB          0x0007  // Direct 32-bit reference to the symbols virtual address, base not included
#define IMAGE_REL_I386_SEG12            0x0009  // Direct 16-bit reference to the segment-selector bits of a 32-bit virtual address
#define IMAGE_REL_I386_SECTION          0x000A
#define IMAGE_REL_I386_SECREL           0x000B
#define IMAGE_REL_I386_TOKEN            0x000C  // clr token
#define IMAGE_REL_I386_SECREL7          0x000D  // 7 bit offset from base of section containing target
#define IMAGE_REL_I386_REL32            0x0014  // PC-relative 32-bit reference to the symbols virtual address

/***
 *	IsRelocated
 *	Verify that the addr. of instruction (or operand) is in the relocation table..
 ***/
BOOL IsRelocated(DWORD address, PEInfo *pPEInfo, short size)
{
	DWORD dwRelocAddr = pPEInfo->ImageNtHeaders->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].VirtualAddress;
	DWORD dwRelocSize = pPEInfo->ImageNtHeaders->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].Size;

	if (dwRelocAddr == 0)
	{
		return FALSE;
	} 

	LPVOID relocAddr = (LPVOID)(RvaToOffset(dwRelocAddr, pPEInfo->ImageNtHeaders) + (DWORD)pPEInfo->FileBase);
	LPVOID baseAddr = relocAddr;

	// relocAddr is the pointer to [X]
	DWORD currentSize = 0;

	DWORD offset = address - pPEInfo->ImageNtHeaders->OptionalHeader.ImageBase;

	while(((DWORD)relocAddr - (DWORD)baseAddr) < dwRelocSize) {
		DWORD*	dwRelocation = (DWORD*)relocAddr;
		WORD*	wRelocation = (WORD*)relocAddr+8;	// base addr + reloc addr

		for(DWORD i = 0; i < dwRelocation[1]; i++) {
			unsigned int relocType = (wRelocation[i] & 0xF000) >> 24;
			unsigned int addr = (wRelocation[i] & 0x0FFF);
			if (offset <= (addr + dwRelocation[0]) && (offset+size) >= (addr + dwRelocation[0])) {
				return TRUE;
			}
		}

		relocAddr = (LPVOID)((DWORD)relocAddr + dwRelocation[0]);	// next ptr 
	}

	return FALSE;
}
