/**
 **/
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "resource.h"

static BOOL IsPaddingX(char *ptr)
{
	BOOL bResult = FALSE;

	switch(tolower(*ptr)) 
	{
		case 'p':
		case 'a':
		case 'd':
		case 'i':
		case 'n':
		case 'g':
		case 'x':
				bResult = TRUE;
			break;
		default:
			break;
	}

	return bResult;
}

static DWORD TryChangeManifest(PEInfo *pInfo, PIMAGE_RESOURCE_DIRECTORY_ENTRY pHeader, PIMAGE_RESOURCE_DIRECTORY pResDirectory, PIMAGE_RESOURCE_DIRECTORY pBase, PIMAGE_RESOURCE_DIRECTORY_ENTRY pDirEntry)
{
	//char tmp[80];

	PIMAGE_RESOURCE_DATA_ENTRY pDataEntry = 
		(PIMAGE_RESOURCE_DATA_ENTRY)(pDirEntry->OffsetToData + (DWORD) pBase);

#ifdef _CONSOLE
	printf("\nFound Data at: %x:Size %x", pDataEntry->OffsetToData, pDataEntry->Size);
#endif

	char *ptr = (char*)(RvaToOffset(pDataEntry->OffsetToData, pInfo->ImageNtHeaders) + (DWORD)pInfo->FileBase);

#ifdef _CONSOLE
	printf("\nManifest %s", ptr);
#endif

	DWORD extraSpace = 0;

	while(IsPaddingX(&ptr[pDataEntry->Size+extraSpace]))
		extraSpace += 1;

#ifdef _CONSOLE
	printf("\nExtra space to expand manifest: %d", extraSpace);
#endif

	char *manifestPtr = new char[pDataEntry->Size+1];

	memset(manifestPtr, 0, pDataEntry->Size+1);
	memcpy(manifestPtr, ptr, pDataEntry->Size);

	char *beginTag = "<requestedPrivileges>";
	char *endTag = "</requestedPrivileges>";

	char *ptrRequestedPrivileges = strstr(manifestPtr, beginTag);
	char *endRequestedPrivileges = strstr(manifestPtr, endTag);
	char *newManifest = new char[pDataEntry->Size+100];

	memset(newManifest, 0, pDataEntry->Size+100);

	if (ptrRequestedPrivileges != NULL) {
		strncpy(newManifest, manifestPtr, ptrRequestedPrivileges - manifestPtr);
		strcat(newManifest, beginTag);
		// embed manifest
		strcat(newManifest, "<requestedExecutionLevel level=\"highestAvailable\" uiAccess=\"false\"/>");
		strcat(newManifest, endRequestedPrivileges);
	}

	DWORD result;

	if (strlen(newManifest) > (pDataEntry->Size + extraSpace)) {
		#ifdef _CONSOLE
			printf("\nUnable to change the manifest");
		#endif

		result = __MANIFEST_ERROR__;
	} else if (strlen(newManifest) != 0) {
		pDataEntry->Size = (DWORD)strlen(newManifest);	// Change current size of manifest
		memcpy(ptr, newManifest, pDataEntry->Size);		// Change manifest!
	
		result = __MANIFEST_OK__;
	} else {
		result = __MANIFEST_ERROR__;
	}

	if (manifestPtr) delete manifestPtr;
	if (newManifest) delete newManifest;

	return result;
}

static DWORD ExploreResourceSubDir(PEInfo *pInfo, PIMAGE_RESOURCE_DIRECTORY_ENTRY pHeader, PIMAGE_RESOURCE_DIRECTORY pResDirectory, PIMAGE_RESOURCE_DIRECTORY pBase)
{
	PIMAGE_RESOURCE_DIRECTORY_ENTRY pDirEntry =
		(PIMAGE_RESOURCE_DIRECTORY_ENTRY)((DWORD)(pResDirectory) + 0x10);

	int entries = pResDirectory->NumberOfIdEntries + pResDirectory->NumberOfNamedEntries;

	if (entries > 0)
		for(int i=0; i < entries; i++) {
			if (pDirEntry[i].DataIsDirectory)
				return ExploreResourceSubDir(pInfo, &pDirEntry[i], (PIMAGE_RESOURCE_DIRECTORY)(pDirEntry[i].OffsetToDirectory + (DWORD)pBase), pBase);
			else
				return TryChangeManifest(pInfo, pHeader, pResDirectory, pBase, &pDirEntry[i]);
	}

	return __MANIFEST_ERROR__;
}

static DWORD ExploreResourceDir(PEInfo *pInfo, PIMAGE_RESOURCE_DIRECTORY pResDirectory)
{
	PIMAGE_RESOURCE_DIRECTORY_ENTRY pDirEntry =
		(PIMAGE_RESOURCE_DIRECTORY_ENTRY)((DWORD)(pResDirectory) + 0x10);

	int entries = pResDirectory->NumberOfIdEntries + pResDirectory->NumberOfNamedEntries;

	for(int i=0; i < entries; i++) {
		if (pDirEntry[i].Id == 0x18 && pDirEntry[i].DataIsDirectory)
				return ExploreResourceSubDir(
					pInfo, 
					&pDirEntry[i], 
					(PIMAGE_RESOURCE_DIRECTORY)(pDirEntry[i].OffsetToDirectory + (DWORD)pResDirectory),
					pResDirectory);

		
	}

	return __MANIFEST_ERROR__;
}

/***
 *	Explorer a resource Section
 ***/
static DWORD ExploreResourceSection(PEInfo *pInfo, PIMAGE_SECTION_HEADER pRsrc)
{
	PIMAGE_RESOURCE_DIRECTORY pResDirectory = (PIMAGE_RESOURCE_DIRECTORY)RvaToOffset(pRsrc->VirtualAddress, pInfo->ImageNtHeaders);

	pResDirectory = (PIMAGE_RESOURCE_DIRECTORY)((DWORD)pResDirectory + (DWORD)pInfo->FileBase);

	return ExploreResourceDir(pInfo, pResDirectory);
}
/***
 *	ChangeManifest
 *	Try to identify the resource with manifest, and to change adding privilege for UAC
 ***/
DWORD ChangeManifest(PEInfo *pInfo)
{
	if (pInfo == NULL)
	{
		return __RESOURCE_GENERIC_ERROR__;
	}

	if (pInfo->ImageOptionalHeader->DataDirectory[2].VirtualAddress == 0)
	{	// 
		return __RESOURCE_NOT_AVAILABLE__;
	}

	PIMAGE_SECTION_HEADER pSection = IMAGE_FIRST_SECTION(pInfo->ImageNtHeaders);

	for(int i=0; i < pInfo->ImageFileHeader->NumberOfSections; i++) {
		if (strcmp((char*)pSection[i].Name, ".rsrc") == 0) {
			// Explore section
			return ExploreResourceSection(pInfo, &pSection[i]);
		}
	}

	return __MANIFEST_NOT_AVAILABLE__;
}